
#include "Flash.h"

/* Intel Strata 28F640K3 in x32 access */
static sector_tab_t sector_INT28F640K3[] =
{
	{ 64, 0x20000}, { 0, 0}
};

/* Intel Strata 28F128K3 in x32 access */
static sector_tab_t sector_INT28F128K3[] =
{
	{ 128, 0x20000}, { 0, 0}
};

/* Intel Strata 28F256K3 in x32 access */
static sector_tab_t sector_INT28F256K3[] =
{
	{ 256, 0x20000}, { 0, 0}
};

Descriptor DeviceTable[] =
{
  0x0000, 0x00, "unknown", 0x00000000, NULL,
  0x8801, 0x89, "Intel 28F640K3C", 0x800000, sector_INT28F640K3,
  0x8802, 0x89, "Intel 28F128K3C", 0x1000000, sector_INT28F128K3,
  0x8803, 0x89, "Intel 28F256K3C", 0x2000000, sector_INT28F256K3,
};
 

void f_Wait(unsigned long times);

unsigned int FlashBase=FLASHBASE;

Descriptor * GetFlashDevice(Descriptor * DeviceData)
{
  int i;
  Descriptor *x;
  
  for(i = 0; i < (sizeof(DeviceTable) / sizeof(Descriptor)); i++)
  {
    if((DeviceTable[i].Device  == DeviceData->Device) && (DeviceTable[i].Manufactur == DeviceData->Manufactur))
    {
    	return(x=&DeviceTable[i]); //???????????????? x notwendig
	}
  }
  
  return(&DeviceTable[0]);  
}      


void FlashStatus(Descriptor * CurrentDevice)
{
  unsigned long Result;
  
  *(volatile unsigned long *) FlashBase = 0x00900090;
  Result = *(volatile unsigned long *) FlashBase;
  (&CurrentDevice[0])->Manufactur = (char) Result;
  (&CurrentDevice[1])->Manufactur = (char) (Result>>16);
  
  *(volatile unsigned long *) (FlashBase+4) = 0x00900090;
  Result = *(volatile unsigned long *) (FlashBase+4);
  (&CurrentDevice[0])->Device = (unsigned short) *(volatile unsigned long *) (FlashBase+4);
  (&CurrentDevice[1])->Device = (unsigned short) ((*(volatile unsigned long *) (FlashBase+4))>>16); 
  
  ResetFlash();
}

void UnlockFlashBlock(unsigned long Address)
{
	*(volatile unsigned long *) Address = 0x00600060;
	*(volatile unsigned long *) Address = 0x00D000D0;
	
	while((ReadFlashStatusReg(Address) & 0x00800080) != 0x00800080);
}

void UnlockFlashBlockAll(void)
{
	unsigned long i = 0, n, Address;
	Descriptor CurrentDevice[2], *FlashDevice;

	FlashStatus(CurrentDevice);
	FlashDevice = GetFlashDevice(&CurrentDevice[0]);
	
	while(FlashDevice->SectorTable[i].n != 0)
	{
		for(n = FlashDevice->SectorTable[i].n; n>0; n--)
		{ 
			Address = FlashBase + (n-1) * FlashDevice->SectorTable[i].size * 2;	// due to 32-Bit wide organisation
			UnlockFlashBlock(Address);			
		}
		
		i++;
	}
	
	ResumeFlash();
}

unsigned long ReadFlashStatusReg(unsigned long Address)
{
	*(volatile unsigned long *) Address = 0x00700070;
	
	return(*(volatile unsigned long *) Address);
}

void ClearFlashStatusReg(void)
{
	*(volatile unsigned long *) FlashBase = 0x00500050;
}

void ResumeFlash(void)
{
	*(volatile unsigned long *) FlashBase = 0x00D000D0;
}

unsigned long ReadFlash(unsigned long Address)
{
	return ( * ((volatile unsigned long *) Address) );
}

void WriteFlash(unsigned long Address, unsigned long Code)
{
	*((volatile unsigned long *) Address) = Code;
}

unsigned long ProgFlash(unsigned long Address, unsigned long Code)
{

	ClearFlashStatusReg();

	*(volatile unsigned long *) Address = 0x00400040;
	*(volatile unsigned long *) Address = Code;
	
	while((ReadFlashStatusReg(Address) & 0x00800080) != 0x00800080);
	
	*(volatile unsigned long *) Address = 0xFFFFFFFF;
  
   return(SUCCESS);
}    

unsigned long EraseFlash(void)
{
	unsigned long i = 0, n, Address;
	Descriptor CurrentDevice[2], *FlashDevice;

	FlashStatus(CurrentDevice);
	FlashDevice = GetFlashDevice(&CurrentDevice[0]);
	
	if(CurrentDevice[0].Device != CurrentDevice[1].Device) return FAIL;

	while(FlashDevice->SectorTable[i].n != 0)
	{
		for(n = FlashDevice->SectorTable[i].n; n>0; n--)
		{ 
			ClearFlashStatusReg();
			WriteFlash(FlashBase + (n-1) * FlashDevice->SectorTable[i].size * 2, 0x00200020);
			WriteFlash(FlashBase + (n-1) * FlashDevice->SectorTable[i].size * 2, 0x00D000D0);
			
			while((ReadFlashStatusReg(FlashBase + (n-1) * FlashDevice->SectorTable[i].size * 2) & 0x00800080) != 0x00800080);
		}
		i++;
	}
	
	ResumeFlash();
	
	return(SUCCESS);
}

unsigned long ResetFlash(void)
{
	*((unsigned long *) FlashBase) = 0xFFFFFFFF;
	
	return(0);
}

void UnprotectCSBOOT(void)
{
} 

void ProtectCSBOOT(void)
{
}

unsigned long FlashTest(unsigned long Lines)
{
 	unsigned long Status, *Flash, Code, ulTmp, Address, Test, i;
 	char BUF[20];
 
 // check whether flash is not empty
 
 	Status = 0;
 	Address = 0;
 	
 	if(ReadFlash(FlashBase) != 0xFFFFFFFF) Status++;
 	
	for(i=0; i<Lines-1; i++)
	{
		if(ReadFlash(FlashBase+Address) != 0xFFFFFFFF) Status++;
		Address = Address << 1;
	} 

	if(Status) 
	{
		EraseFlash();
	}
	
	Status = 0;
	
// end empty check

	Code = 0x00020002;
	Address = 4;

	ProgFlash(FlashBase+0x00000000,0x00010001);	

	for(i=0; i<Lines; i++)
	{
		ProgFlash(FlashBase+Address, Code);
		Address = Address << 1;
		Code = Code + 0x00010001;
	}	

	ResumeFlash();

	Status = 0;
	Address = 0;
	Code = 0;
	Flash = (unsigned long *) FlashBase;

	if(Flash[Address] != (Code+=0x00010001))
	{
		Status++;
		//puts("A0 corrupted\n\r");
	}

	Address++;
	
	for(ulTmp=0; ulTmp<Lines; ulTmp++)
	{
		if((Test=Flash[Address]) != (Code+=0x00010001))
		{
			Status++;	
			printf("A %d\n\r",ulTmp);
		}
		Address = Address <<1;
	}	

  return(Status);

}

unsigned char FlashTestProcedure(void)
{
	Descriptor CurrentDevice[2], *FlashDevice;
	unsigned long ul;
	int iTmp;

	FlashBase = 0xFE000000;

	UnlockFlashBlockAll();

	FlashStatus(CurrentDevice);
  
	printf("FLASH U5\t\t");
	FlashDevice = GetFlashDevice(&CurrentDevice[0]);
	
	printf(FlashDevice->DeviceName);
	printf("\n\r");
 
	printf("FLASH U6\t\t");
	FlashDevice = GetFlashDevice(&CurrentDevice[1]);
	printf(FlashDevice->DeviceName);
	printf("\n\r");
  
	printf("FLASH\t\t\ttesting");
	
	switch(FlashDevice->Capacity)
	{
		case 0x00800000:	ul = FlashTest(22);	// 16MByte (A23..A2)
							break;

		case 0x01000000:	ul = FlashTest(23);	// 32MByte (A24..A2)
							break;
							
		case 0x02000000:	ul = FlashTest(24); // 64MByte (A25..A2)
							break;							
	}
			
  if(!ul)
  {
	printf("\b\b\b\b\b\b\berasing      ");
	EraseFlash();
	printf("\b\b\b\b\b\b\b\b\b\b\b\b\bOK      \n\r");   
    return(0);
  }  
  else
  {
    printf("\b\b\b\b\b\b\b\b\b\b\b\b\bERROR        \n\r");
	return(1);
	}
}

  
void f_Wait(unsigned long times)
{
  unsigned long n;

  for(n = 0; n < times; n++) {};
}
  