#ifndef _FLASH_H_
#define _FLASH_H_

#define FLASHBASE 0xFE000000

#define PASS    0
#define SUCCESS 0
#define FAIL    1


// Manufacturer IDs
#define MANUFACTURER_AMD	0x01
#define MANUFACTURER_MBM	0x04	/* Fujitsu */
#define MANUFACTURER_AT		0x1F	/* Atmel */
#define MANUFACTURER_STM	0x20	/* STMicroelectronix */
#define MANUFACTURER_HY		0xAD	/* Hyundai */
#define MANUFACTURER_SST	0xBF	// SST
#define MANUFACTURER_MX		0xC2	/* Macronix */
#define MANUFACTURER_INT	0x89	// Intel


#ifndef NULL
   #define NULL ((void *) 0)
#endif

typedef struct
{
	unsigned int n;		/* number of sectors */
	unsigned int size;	/* size of sector */
} sector_tab_t;

typedef struct
{
  unsigned short	Device;
  char				Manufactur;
  char				*DeviceName;
  unsigned long		Capacity;
  sector_tab_t		*SectorTable;
} Descriptor;


typedef struct
{
	unsigned int addr;
	unsigned int length;
	unsigned int usage;
} sector_t;


#define Am29F200AT	0x2251
#define Am29F400AB	0x22AB
#define Am29F800B	0x225B

#define Am29LV200B	0x22BF
#define Am29LV400B	0xffBA
#define Am29LV800B	0x22DA

extern unsigned int FlashBase;

unsigned char FlashTestProcedure(void);

void UnprotectCSBOOT(void);
void ProtectCSBOOT(void);
unsigned long EraseFlash(void);
unsigned long ResetFlash(void);
unsigned long ProgFlash(unsigned long Address, unsigned long Code);
void FlashStatus(Descriptor *);
void BurstFlashStatus(Descriptor * CurrentDevice);
Descriptor * GetFlashDevice(Descriptor * DeviceData);
unsigned long FlashTest(unsigned long Lines);
unsigned long ReadFlash(unsigned long Address);
void WriteFlash(unsigned long Address, unsigned long Code);

unsigned long ReadFlashStatusReg(unsigned long Address);
void ClearFlashStatusReg(void);
void ResumeFlash(void);
void UnlockFlashBlock(unsigned long Address);
void UnlockFlashBlockAll(void);

#endif