/*
 * File:		sysinit.c
 * Purpose:		Verdi Reset Configuration
 *
 * Notes:
 *
 */

#include "src/include/common.h"
#include "src/include/psc_uart.h"

/********************************************************************/

void fbcs_init(void);
void sdramc_init(void);

/********************************************************************/
void
mcf548x_init (void)
{
	extern char __DATA_ROM[];
	extern char __DATA_RAM[];
	extern char __DATA_END[];
	extern char __BSS_START[];
	extern char __BSS_END[];
	extern uint32 VECTOR_TABLE[];
	extern uint32 __VECTOR_RAM[];

	uint32 n;
	uint8 *dp, *sp;
	
	psc_uart_init(TERMINAL_PORT, TERMINAL_BAUD);
	fbcs_init();
	sdramc_init();

	/* Copy the vector table to RAM */
	if (__VECTOR_RAM != VECTOR_TABLE)
	{
		for (n = 0; n < 256; n++)
			__VECTOR_RAM[n] = VECTOR_TABLE[n];
	}
    mcf5xxx_wr_vbr((uint32)__VECTOR_RAM);

	/* Move initialized data from ROM to RAM. */
	if(1)
	if (__DATA_ROM != __DATA_RAM)
	{
		dp = (uint8 *)__DATA_RAM;
		sp = (uint8 *)__DATA_ROM;
		n = __DATA_END - __DATA_RAM;
		while (n--)
			*dp++ = *sp++;
	}
 
	/* Zero uninitialized data */
	if (__BSS_START != __BSS_END)
	{
		sp = (uint8 *)__BSS_START;
		n = __BSS_END - __BSS_START;
		while (n--)
			*sp++ = 0;
	}
}
/********************************************************************/
void
fbcs_init (void)
{
	/* Flash */
	MCF_FBCS_CSAR0 = MCF_FBCS_CSAR_BA(FLASH_ADDRESS);
	
	MCF_FBCS_CSCR0 = (MCF_FBCS_CSCR_PS_32
					| MCF_FBCS_CSCR_AA
					| MCF_FBCS_CSCR_ASET(1)
					| MCF_FBCS_CSCR_WS(6));					
					
	MCF_FBCS_CSMR0 = (MCF_FBCS_CSMR_BAM_32M 
					| MCF_FBCS_CSMR_V);

    /* CF-Card */              
	MCF_FBCS_CSAR1 = MCF_FBCS_CSAR_BA(CFCARD_ADDRESS);
	MCF_FBCS_CSCR1 = (MCF_FBCS_CSCR_PS_16
					| MCF_FBCS_CSCR_AA
					| MCF_FBCS_CSCR_ASET(1)
					| MCF_FBCS_CSCR_WS(15)
					| MCF_FBCS_CSCR_BEM);					
	MCF_FBCS_CSMR1 = (MCF_FBCS_CSMR_BAM_16M 
					| MCF_FBCS_CSMR_V);

	/* XPLD */
	MCF_FBCS_CSAR2 = MCF_FBCS_CSAR_BA(XPLD_ADDRESS);	
	MCF_FBCS_CSCR2 = (MCF_FBCS_CSCR_PS_16
					| MCF_FBCS_CSCR_AA
					| MCF_FBCS_CSCR_ASET(1)
					| MCF_FBCS_CSCR_WS(2));					
	MCF_FBCS_CSMR2 = (MCF_FBCS_CSMR_BAM_16M 
					| MCF_FBCS_CSMR_V);					             
					
}  
/********************************************************************/
void
sdramc_init (void)
{
	/* Initialize DDR DIMMs on the Verdi board */
	
	/*
	 * Check to see if the SDRAM has already been initialized
	 * by a run control tool
	 */
	if (!(MCF_SDRAMC_SDCR & MCF_SDRAMC_SDCR_REF))
	{

if(0)
{
		MCF_SDRAMC_SDRAMDS = 0x000002AA;
		MCF_SDRAMC_CS0CFG = 0x0000001A;
		MCF_SDRAMC_CS1CFG = 0x00000000;
		MCF_SDRAMC_CS2CFG = 0x00000000;
		MCF_SDRAMC_CS3CFG = 0x00000000;
		
		MCF_SDRAMC_SDCFG1 = 0x53722930;
		MCF_SDRAMC_SDCFG2 = 0x24330000;
		
		MCF_SDRAMC_SDCR = 0xE10F0002;
		MCF_SDRAMC_SDMR = 0x40010000;
		MCF_SDRAMC_SDMR = 0x05890000;
		MCF_SDRAMC_SDCR = 0xE10F0002;
		MCF_SDRAMC_SDCR = 0xE10F0004;
		MCF_SDRAMC_SDCR = 0xE10F0004;
		MCF_SDRAMC_SDMR = 0x01890000;
		MCF_SDRAMC_SDCR = 0x710F0F00;	
}
else
{
		/* Basic configuration and initialization */
		
		MCF_SDRAMC_SDRAMDS = (0
			| MCF_SDRAMC_SDRAMDS_SB_E(MCF_SDRAMC_SDRAMDS_DRIVE_8MA)
			| MCF_SDRAMC_SDRAMDS_SB_C(MCF_SDRAMC_SDRAMDS_DRIVE_8MA)
			| MCF_SDRAMC_SDRAMDS_SB_A(MCF_SDRAMC_SDRAMDS_DRIVE_8MA)
			| MCF_SDRAMC_SDRAMDS_SB_S(MCF_SDRAMC_SDRAMDS_DRIVE_8MA)
			| MCF_SDRAMC_SDRAMDS_SB_D(MCF_SDRAMC_SDRAMDS_DRIVE_8MA) ); // 0x0000_02AA
				
		MCF_SDRAMC_CS0CFG = (0
			| MCF_SDRAMC_CSnCFG_CSBA(SDRAM_ADDRESS)
			| MCF_SDRAMC_CSnCFG_CSSZ(MCF_SDRAMC_CSnCFG_CSSZ_128MBYTE) ); // 0x0000_001A
				
		MCF_SDRAMC_SDCFG1 = (0
			| MCF_SDRAMC_SDCFG1_SRD2RW(5)
			| MCF_SDRAMC_SDCFG1_SWT2RD(3)
			| MCF_SDRAMC_SDCFG1_RDLAT(7)
			| MCF_SDRAMC_SDCFG1_ACT2RW(2)
			| MCF_SDRAMC_SDCFG1_PRE2ACT(2)
			| MCF_SDRAMC_SDCFG1_SRD2RW(5)
			| MCF_SDRAMC_SDCFG1_REF2ACT(9)
			| MCF_SDRAMC_SDCFG1_WTLAT(3)   ); // 0x5372_2930
			
		MCF_SDRAMC_SDCFG2 = (0
			| MCF_SDRAMC_SDCFG2_BRD2PRE(2)
			| MCF_SDRAMC_SDCFG2_BWT2RW(4)
			| MCF_SDRAMC_SDCFG2_BRD2WT(3)
			| MCF_SDRAMC_SDCFG2_BL(3)      ); // 0x2433_0000
				
		/* Precharge and enable write to SDMR */
		MCF_SDRAMC_SDCR = (0
			| MCF_SDRAMC_SDCR_MODE_EN
			| MCF_SDRAMC_SDCR_CKE
			| MCF_SDRAMC_SDCR_DDR
			| MCF_SDRAMC_SDCR_MUX(0x01)
			| MCF_SDRAMC_SDCR_RCNT(0xF)
			| MCF_SDRAMC_SDCR_IPALL	     ); // 0xE10F_0002
		
		/* Write extended mode register */
		MCF_SDRAMC_SDMR = (0
			| MCF_SDRAMC_SDMR_BNKAD_LEMR
			| MCF_SDRAMC_SDMR_AD(0x0)
			| MCF_SDRAMC_SDMR_CMD        ); // 0x4001_0000
		
		/* Write mode register and reset DLL */ 	
		MCF_SDRAMC_SDMR = (0
			| MCF_SDRAMC_SDMR_BNKAD_LMR
			| MCF_SDRAMC_SDMR_AD(0x588)
			| MCF_SDRAMC_SDMR_CMD        ); // 0x0589_0000
				
		/* Execute a PALL command */
		MCF_SDRAMC_SDCR = (MCF_SDRAMC_SDCR
			| MCF_SDRAMC_SDCR_IPALL      ); // 0xE10F_0002
		
		/* Perform two REF cycles */
		MCF_SDRAMC_SDCR = (MCF_SDRAMC_SDCR		 /*perform first refresh*/
			| MCF_SDRAMC_SDCR_IREF       ); // 0xE10F_0004
		
		MCF_SDRAMC_SDCR = (MCF_SDRAMC_SDCR		 /*perform second refresh*/
			| MCF_SDRAMC_SDCR_IREF       ); // 0xE10F_0004
			
		/* Write mode register and clear reset DLL */
		MCF_SDRAMC_SDMR = (0
			| MCF_SDRAMC_SDMR_BNKAD_LMR
			| MCF_SDRAMC_SDMR_AD(0x188)
			| MCF_SDRAMC_SDMR_CMD        ); // 0x0189_0000
				
		/* Enable auto refresh and lock SDMR */
		MCF_SDRAMC_SDCR = (0 | MCF_SDRAMC_SDCR
			& ~MCF_SDRAMC_SDCR_MODE_EN
			| MCF_SDRAMC_SDCR_REF
			| MCF_SDRAMC_SDCR_DQS_OE(0xF) ); // 0x710F_0006			
}		
	}
}
/********************************************************************/
