/*
 * File:		verdi_leds.c
 * Purpose:		Provide functions for using the LEDs on Verdi
 *
 * Notes:
 */

#include "src/include/common.h"
#include "src/include/verdi_leds.h"

/*********************************************************************
 * Enable the 4 LEDs connected to TOUT[3:0]
 */
static void
verdi_leds_enable(void)
{
	/*
	 * Enable TOUT2 and TOUT3 as Timer pins via the GPIO module
	 */
	MCF_GPIO_PAR_TIMER = (0
		| MCF_GPIO_PAR_TIMER_PAR_TOUT3
		| MCF_GPIO_PAR_TIMER_PAR_TOUT2);

	/*
	 * Now enable TOUT[3:0] as GPIO in the Timer module
	 */
	MCF_GPT_GMS0 = (MCF_GPT_GMS_TMS_GPIO | MCF_GPT_GMS_GPIO_OUTHI);
	MCF_GPT_GMS1 = (MCF_GPT_GMS_TMS_GPIO | MCF_GPT_GMS_GPIO_OUTHI);         
	MCF_GPT_GMS2 = (MCF_GPT_GMS_TMS_GPIO | MCF_GPT_GMS_GPIO_OUTHI);
	MCF_GPT_GMS3 = (MCF_GPT_GMS_TMS_GPIO | MCF_GPT_GMS_GPIO_OUTHI);
}
/*********************************************************************
 * Display the lower 4 bits of 'number' on the 4 LEDs connected to 
 * TOUT[3:0]
 *
 *  LED:  D29    D28    D27    D26
 *  PIN:  TOUT3  TOUT2  TOUT1  TOUT0
 *  BIT:  3      2      1      0
 */
void
verdi_leds_display(uint32 number, uint32 delay)
{
	int i;

    verdi_leds_enable();
    
	/*
	 * Turn on the appropriate LEDs
	 */
	for (i = 0; i < 4; i++)
	{
		if (number & 0x01)
		{
			MCF_GPT_GMS(i) = (MCF_GPT_GMS_TMS_GPIO | MCF_GPT_GMS_GPIO_OUTLO);
		}
		number = number >> 1;
	}

	if (delay)
	{
		for (i = 0; i < delay; i++) 
		{
			nop();
		}
	}
    
}
/********************************************************************/
