/**********************************************************************
 *
 * Filename:    memtest.h
 * 
 * Description: Memory-testing module API.
 *
 * Notes:       The memory tests can be easily ported to systems with
 *              different data bus widths by redefining 'datum' type.
 *
 * 
 * Copyright (c) 2000 by Michael Barr.  This software is placed into
 * the public domain and may be used for any purpose.  However, this
 * notice must not be changed or removed and no warranty is either
 * expressed or implied by its publication or distribution.
 **********************************************************************/

#ifndef _memtest_h
#define _memtest_h


/*
 * Define NULL pointer value.
 */
#ifndef NULL
#define NULL  (void *) 0
#endif

/*
 * Set the data bus width.
 */
typedef unsigned long datum;


#define RAM_NO    0xFF
#define RAM_2MB   1
#define RAM_4MB   RAM_2MB + 1
#define RAM_8MB   RAM_4MB + 1
#define RAM_16MB  RAM_8MB + 1
#define RAM_32MB  RAM_16MB + 1
#define RAM_64MB  RAM_32MB + 1
#define RAM_128MB  RAM_64MB + 1
#define RAM_256MB  RAM_128MB + 1


typedef struct SRAMConfig
{
	unsigned char SRAMStatus;
	char * Amount;
	unsigned long Vol;
} SRAMConfig;


/*
 * Function prototypes.
 */
datum   memTestDataBus(volatile datum * address);
datum * memTestAddressBus(volatile datum * baseAddress, unsigned long nBytes);
datum * memTestDevice(volatile datum * baseAddress, unsigned long nBytes);

unsigned char MemStatus(void);

#endif /* _memtest_h */

