/*
 * File:        verdi.h
 * Purpose:     Validation board definitions and memory map information
 *
 * Notes:
 */

#ifndef __PHYCOREMCF548x_H__
#define __PHYCOREMCF548x_H__

/********************************************************************/

/*
 * Debug prints ON (#define) or OFF (#undef)
 */
#define DEBUG

/*
 * System Bus Clock Info
 */

// phyCORE
#define SYSTEM_CLOCK        100     /* system bus frequency in MHz */
#define PERIOD              10.     /* system bus period in ns */


/*
 * Flash Info
 */

/*
 * Serial Port Info
 */
#define TERMINAL_PORT       0       /* PSC channel used as terminal */
#define TERMINAL_BAUD       19200   /* 19200 Baud */
#undef  HARDWARE_FLOW_CONTROL       /* Flow control ON or OFF */

/*
 * Ethernet Port Info
 */
#define FEC_PHY0            (0x01)
#define FEC_PHY1            (0x01)
#define FEC_PHY(x)          ((x == 0) ? FEC_PHY0 : FEC_PHY1)

/*
 * XPLD (FB_CS2 16-Bit port size)
 */

#define XPLD_ADDRESS		0xF0000000
#define CFCARD_ADDRESS		0xF1000000

#define pREG_0_DAT	(*(volatile unsigned short *) (XPLD_BASE + 0x800))
#define pREG_0_DIR	(*(volatile unsigned short *) (XPLD_BASE + 0x802))
#define pREG_1_DAT	(*(volatile unsigned short *) (XPLD_BASE + 0x804))
#define pREG_1_DIR	(*(volatile unsigned short *) (XPLD_BASE + 0x806))
#define pREG_CFG	(*(volatile unsigned short *) (XPLD_BASE + 0x808))

#define LED		0x0001

#define LED_Init	(pREG_0_DIR |= LED)
#define LED_On		(pREG_0_DAT &= LED)
#define LED_Off		(pREG_0_DAT |= LED)
#define LED_Toggle	((pREG_0_DAT & LED) ? LED_Off : LED_On)

/*
 * Memory map definitions from linker command files
 */
 
extern uint8 __MBAR[];
extern uint8 __SDRAM[];
extern uint8 __SDRAM_SIZE[];
extern uint8 __SYS_SRAM[];
extern uint8 __SYS_SRAM_SIZE[];
extern uint8 __CORE_SRAM0[];
extern uint8 __CORE_SRAM0_SIZE[];
extern uint8 __CORE_SRAM1[];
extern uint8 __CORE_SRAM1_SIZE[];
extern uint8 __EXT_SRAM[];
extern uint8 __EXT_SRAM_SIZE[];
extern uint8 __FLASH[];
extern uint8 __FLASH_SIZE[];

/*
 * Memory Map Info
 */
#define MBAR_ADDRESS        (uint32)__MBAR

#define SDRAM_ADDRESS       (uint32)__SDRAM
#define SDRAM_SIZE          (uint32)__SDRAM_SIZE

#define SYS_SRAM_ADDRESS    (uint32)__SYS_SRAM
#define SYS_SRAM_SIZE       (uint32)__SYS_SRAM_SIZE

#define CORE_SRAM0_ADDRESS  (uint32)__CORE_SRAM0
#define CORE_SRAM0_SIZE     (uint32)__CORE_SRAM0_SIZE

#define CORE_SRAM1_ADDRESS  (uint32)__CORE_SRAM1
#define CORE_SRAM1_SIZE     (uint32)__CORE_SRAM1_SIZE

#define EXT_SRAM_ADDRESS    (uint32)__EXT_SRAM
#define EXT_SRAM_SIZE       (uint32)__EXT_SRAM_SIZE

#define FLASH_ADDRESS       (uint32)__FLASH
#define FLASH_SIZE          (uint32)__FLASH_SIZE

/********************************************************************/

#endif /* __PHYCOREMCF548x_H__ */
